﻿using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Share.Enums.Order;
using Hotline.Users;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using XF.Domain.Exceptions;

namespace Hotline.Application.Orders;

public class OrderApplication : IOrderApplication, IScopeDependency
{
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly IOrderRepository _orderRepository;

    public OrderApplication(
        IWorkflowDomainService workflowDomainService,
        IOrderRepository orderRepository
        )
    {
        _workflowDomainService = workflowDomainService;
        _orderRepository = orderRepository;
    }

    /// <summary>
    /// 工单办理(每个节点都会触发)
    /// </summary>
    public async Task OrderManageAsync(EOrderStatus status, EFlowAssignType assignType, bool isCountersignEnd, bool isCountersignStart,
        string orderId, DateTime currentStepTime, string CurrentStepName, IEnumerable<string> handlers,
        CancellationToken cancellationToken)
    {
        var order = await _orderRepository.GetAsync(orderId, cancellationToken);
        if (order == null)
            throw new UserFriendlyException($"无效工单编号, orderId: {orderId}", "无效工单编号");
        if (order.Status is EOrderStatus.Filed) return;

        //order.Assign(assignType, handlers);
        await AssignAsync(order, assignType, handlers, cancellationToken);

        //1.如果order未处于会签中，则判断是否发起会签（isstartCountersign） 2.如果处于会签中，则判断会签是否结束（isCountersignEnd）
        if (order.Status is EOrderStatus.Countersigning && isCountersignEnd)
        {
            order.Status = EOrderStatus.WaitForSign;
        }
        else if (order.Status is not EOrderStatus.Countersigning && isCountersignStart)
        {
            order.Status = EOrderStatus.Countersigning;
        }

        order.CurrentStepTime = currentStepTime;
        order.CurrentStepName = CurrentStepName;
        order.Status = status;

        await _orderRepository.UpdateAsync(order, cancellationToken);
    }

    private async Task AssignAsync(Order order, EFlowAssignType assignType, IEnumerable<string> handlers, CancellationToken cancellationToken)
    {
        switch (assignType)
        {
            case EFlowAssignType.Org:
                order.AssignOrgCodes.AddRange(handlers);
                order.AssignOrgCodes = order.AssignOrgCodes.Distinct().ToList();
                break;
            case EFlowAssignType.User:
                order.AssignUserIds.AddRange(handlers);
                order.AssignUserIds = order.AssignUserIds.Distinct().ToList();
                break;
            case EFlowAssignType.Role:
                var users =
                    AssignRoles.AddRange(handlers);
                AssignRoles = AssignRoles.Distinct().ToList();
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(type), type, null);
        }
    }
}